 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

/*
 * Nautilus
 *
 * Copyright (C) 1999, 2000, 2001 Eazel, Inc.
 *
 * Nautilus is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Nautilus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author:  James Willcox  <jwillcox@gnome.org>
 *
 * This is a sidebar displaying emblems which can be dragged onto files to
 * set/unset the chosen emblem for that particular file.
 *
 */

#include <config.h>
#include "nautilus-emblem-view.h"

#include <eel/eel-gtk-macros.h>
#include <eel/eel-string.h>
#include <eel/eel-wrap-table.h>
#include <eel/eel-labeled-image.h>
#include <eel/eel-graphic-effects.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gtk/gtk.h>
#include <libgnomeui/gnome-uidefs.h>
#include <libgnomevfs/gnome-vfs-utils.h>
#include <libnautilus-private/nautilus-customization-data.h>
#include <libnautilus-private/nautilus-icon-factory.h>

struct NautilusEmblemViewDetails {
	GtkVBox *container;

	gchar *drag_keyword;
};

#define ERASE_EMBLEM_KEYWORD			"erase"
#define STANDARD_EMBLEM_HEIGHT			52
#define EMBLEM_LABEL_SPACING			2

static void     nautilus_emblem_view_class_init      (NautilusEmblemViewClass *object_klass);
static void     nautilus_emblem_view_instance_init   (NautilusEmblemView *object);
static void     nautilus_emblem_view_dispose         (GObject     *object);
static void     nautilus_emblem_view_finalize        (GObject     *object);

static GtkTargetEntry drag_types[] = {
	{"property/keyword", 0, 0 }
};


BONOBO_CLASS_BOILERPLATE (NautilusEmblemView, nautilus_emblem_view,
			  NautilusView, NAUTILUS_TYPE_VIEW)

/* utility routine to strip the extension from the passed in string */
static char*
strip_extension (const char* string_to_strip)
{
	char *result_str, *temp_str;
	if (string_to_strip == NULL)
		return NULL;
	
	result_str = g_strdup(string_to_strip);
	temp_str = strrchr(result_str, '.');
	if (temp_str)
		*temp_str = '\0';
	return result_str;
}

static void
nautilus_emblem_view_drag_data_get_cb (GtkWidget *image,
					GdkDragContext *context,
					GtkSelectionData *data,
					guint info,
					guint time,
					NautilusEmblemView *emblem_view)
{
	gtk_selection_data_set (data, data->target, 8,
				emblem_view->details->drag_keyword,
				strlen (emblem_view->details->drag_keyword));
}

static void
nautilus_emblem_view_drag_begin_cb (GtkWidget *widget,
				     GdkDragContext *drag_context,
				     NautilusEmblemView *emblem_view)
{
	gchar *keyword;

	keyword = g_object_get_data (G_OBJECT (widget), "emblem-keyword");

	emblem_view->details->drag_keyword = keyword;
}

static void
nautilus_emblem_view_enter_notify_cb (GtkWidget *widget,
				       NautilusEmblemView *emblem_view)
{
	GdkPixbuf *pixbuf;
	EelLabeledImage *image;

	pixbuf = g_object_get_data (G_OBJECT (widget), "prelight-pixbuf");
	image = g_object_get_data (G_OBJECT (widget), "labeled-image");

	eel_labeled_image_set_pixbuf (EEL_LABELED_IMAGE (image), pixbuf);
}

static void
nautilus_emblem_view_leave_notify_cb (GtkWidget *widget,
				       NautilusEmblemView *emblem_view)
{
	GdkPixbuf *pixbuf;
	EelLabeledImage *image;

	pixbuf = g_object_get_data (G_OBJECT (widget), "original-pixbuf");
	image = g_object_get_data (G_OBJECT (widget), "labeled-image");

	eel_labeled_image_set_pixbuf (EEL_LABELED_IMAGE (image), pixbuf);
}

static GtkWidget *
nautilus_emblem_view_get_content (NautilusEmblemView *emblem_view)
{
	NautilusCustomizationData *customization_data;
	GtkWidget *emblems_table, *image, *scroller;
	GtkWidget *erase_widget = NULL;
	char *emblem_name;
	GdkPixbuf *pixbuf;
	GtkWidget *event_box;
	char *label;

	/* The emblems wrapped table */
	scroller = eel_scrolled_wrap_table_new (TRUE, &emblems_table);

	gtk_container_set_border_width (GTK_CONTAINER (emblems_table), GNOME_PAD);
	
	gtk_widget_show (scroller);

	/* Use nautilus_customization to make the emblem widgets */
	customization_data = nautilus_customization_data_new ("emblems", TRUE,
						TRUE,
						NAUTILUS_ICON_SIZE_SMALL,
						NAUTILUS_ICON_SIZE_SMALL);
	
	while (nautilus_customization_data_get_next_element_for_display (customization_data,
									 &emblem_name,
									 &pixbuf,
									 &label) == GNOME_VFS_OK) {	
		GdkPixbuf *prelight_pixbuf;
		gboolean is_erase_widget = FALSE;
		gchar *keyword;

		keyword = strip_extension (emblem_name);
		g_free (emblem_name);


		image = eel_labeled_image_new (label, pixbuf);
		eel_labeled_image_set_fixed_image_height (EEL_LABELED_IMAGE (image), STANDARD_EMBLEM_HEIGHT);
		eel_labeled_image_set_spacing (EEL_LABELED_IMAGE (image), EMBLEM_LABEL_SPACING);
		event_box = gtk_event_box_new ();
		gtk_container_add (GTK_CONTAINER (event_box), image);

		prelight_pixbuf = eel_create_spotlight_pixbuf (pixbuf);
		

		gtk_drag_source_set (event_box, GDK_BUTTON1_MASK, drag_types,
				     G_N_ELEMENTS (drag_types),
				     GDK_ACTION_COPY|GDK_ACTION_MOVE);

		gtk_drag_source_set_icon_pixbuf (event_box, pixbuf);

		g_signal_connect (event_box, "drag-data-get",
			G_CALLBACK (nautilus_emblem_view_drag_data_get_cb),
			emblem_view);
		g_signal_connect (event_box, "drag-begin",
			G_CALLBACK (nautilus_emblem_view_drag_begin_cb),
			emblem_view);
		g_signal_connect (event_box, "enter-notify-event",
			G_CALLBACK (nautilus_emblem_view_enter_notify_cb),
			emblem_view);
		g_signal_connect (event_box, "leave-notify-event",
			G_CALLBACK (nautilus_emblem_view_leave_notify_cb),
			emblem_view);
		
		g_free (label);

		g_object_set_data_full (G_OBJECT (event_box),
					"emblem-keyword",
					keyword, g_free);
		g_object_set_data_full (G_OBJECT (event_box),
					"original-pixbuf",
					pixbuf, g_object_unref);
		g_object_set_data_full (G_OBJECT (event_box),
					"prelight-pixbuf",
					prelight_pixbuf, g_object_unref);
		g_object_set_data (G_OBJECT (event_box),
				   "labeled-image", image);

		if (strcmp (ERASE_EMBLEM_KEYWORD, keyword) == 0) {
			is_erase_widget = TRUE;
			erase_widget = event_box;
		}

		if (!is_erase_widget)
			gtk_container_add (GTK_CONTAINER (emblems_table), event_box);
	}

	/* we want to put the erase icon at the end */
	if (erase_widget != NULL)
			gtk_container_add (GTK_CONTAINER (emblems_table),
					   erase_widget);

	gtk_widget_show_all (emblems_table);

	return scroller;
}

static void
nautilus_emblem_view_instance_init (NautilusEmblemView *emblem_view)
{
	BonoboControl *control;
	GtkWidget *widget;
	
	emblem_view->details = g_new0 (NautilusEmblemViewDetails, 1);

	widget = nautilus_emblem_view_get_content (emblem_view);

	control = bonobo_control_new (widget);
	nautilus_view_construct_from_bonobo_control
				(NAUTILUS_VIEW (emblem_view), control);

}

static void
nautilus_emblem_view_dispose (GObject *object)
{
	NautilusEmblemView *emblem_view;

	emblem_view = NAUTILUS_EMBLEM_VIEW (object);

	if (emblem_view->details) {
		g_free (emblem_view->details);
		emblem_view->details = NULL;
	}
}

static void
nautilus_emblem_view_finalize (GObject *object)
{
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
nautilus_emblem_view_class_init (NautilusEmblemViewClass *object_klass)
{
	GObjectClass *gobject_class;
	
	NautilusEmblemViewClass *klass;

	klass = NAUTILUS_EMBLEM_VIEW_CLASS (object_klass);
	gobject_class = G_OBJECT_CLASS (object_klass);

	gobject_class->finalize = nautilus_emblem_view_finalize;
	gobject_class->dispose  = nautilus_emblem_view_dispose;
}

